# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import  testtools

from ecl.sss.v1 import user

BASIC_EXAMPLE = {
    "user_id": "ecid1000005297",
    "login_id": "userlogin_id",
    "mail_address": "youremail@ntt.com",
    "contract_owner": True,
    "sss_endpoint": "https://sss-jp1-ecl.api.ntt.com:443/api",
    "keystone_endpoint": "https://keystone-jp1-ecl.api.ntt.com:443",
    "keystone_name": "XXXXXXXXXXX",
    "keystone_password": "XXXXXXXXXX",
    "start_time": "2016-01-06 00:44:39",
    "contract_id": "econ1000003351",
    "login_integration": "icp",
    "external_reference_id": "econ1000003351",
    "password": "password",
    "notify_password": True,
    "new_password": "new_password",
}

class TestUser(testtools.TestCase):

    def test_user(self):
        sot = user.User()
        self.assertEqual("users", sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("/users", sot.base_path)
        self.assertEqual('sss', sot.service.service_type)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.put_create)

    def test_make_user(self):
        sot = user.User(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['login_id'], sot.login_id)
        self.assertEqual(BASIC_EXAMPLE['mail_address'], sot.mail_address)
        self.assertEqual(BASIC_EXAMPLE['user_id'], sot.user_id)
        self.assertEqual(BASIC_EXAMPLE['contract_owner'], sot.contract_owner)
        self.assertEqual(BASIC_EXAMPLE['contract_id'], sot.contract_id)
        self.assertEqual(BASIC_EXAMPLE['keystone_name'], sot.keystone_name)
        self.assertEqual(BASIC_EXAMPLE['keystone_endpoint'], sot.keystone_endpoint)
        self.assertEqual(BASIC_EXAMPLE['keystone_password'], sot.keystone_password)
        self.assertEqual(BASIC_EXAMPLE['sss_endpoint'], sot.sss_endpoint)
        self.assertEqual(BASIC_EXAMPLE['password'], sot.password)
        self.assertEqual(BASIC_EXAMPLE['notify_password'], sot.notify_password)
        self.assertEqual(BASIC_EXAMPLE['new_password'], sot.new_password)
        self.assertEqual(BASIC_EXAMPLE['external_reference_id'], sot.external_reference_id)
        self.assertEqual(BASIC_EXAMPLE['login_integration'], sot.login_integration)



# -*- coding: utf-8 -*-


from ecl.network import network_service
from ecl.network.v2.base import NetworkBaseResource
from ecl import resource2
from ecl import exceptions


class InternetService(NetworkBaseResource):

    resource_key = "internet_service"
    resources_key = "internet_services"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/internet_services'

    allow_list = True
    allow_get = True

    _query_mapping = resource2.QueryParameters(
        "description", "id", "minimal_subnet_length",
        "zone", "name", "sort_key", "sort_dir",
    )

    description = resource2.Body("description")
    id = resource2.Body("id")
    minimal_submask_length = resource2.Body("minimal_submask_length")
    zone = resource2.Body("zone")
    name = resource2.Body("name")

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))


class InternetGateway(NetworkBaseResource):

    resource_key = "internet_gateway"
    resources_key = "internet_gateways"
    base_path = '/v2.0/internet_gateways'
    service = network_service.NetworkService()

    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True

    _query_mapping = resource2.QueryParameters(
        "description", "id", "internet_service_id",
        "name", "qos_option_id", "status", "tenant_id",
        "sort_key", "sort_dir",
    )

    description = resource2.Body("description")
    id = resource2.Body("id")
    internet_service_id = resource2.Body("internet_service_id")
    name = resource2.Body("name")
    qos_option_id = resource2.Body("qos_option_id")
    status = resource2.Body("status")
    tenant_id = resource2.Body("tenant_id")

    def wait_for_gateway(self, session, status='ACTIVE', failures=['ERROR'],
                        interval=2, wait=120):
        return resource2.wait_for_status(session, self, status,
                                         failures, interval, wait)

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestApiKey(base.BaseFunctionalTest):

    def test_01_update_api_key(self):
        api_key=self.conn.sss.update_api_key(
            "ecid1000378024"
        )
        self.assertIsInstance(api_key.user_id, six.string_types)
        self.assertIsInstance(api_key.keystone_name, six.string_types)
        self.assertIsInstance(api_key.keystone_password, six.string_types)
        self.assertIsInstance(api_key.status, six.string_types)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.identity.v3 import project

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'description': '1',
    'domain_id': '2',
    'enabled': True,
    'id': IDENTIFIER,
    'name': '5',
}


class TestProject(testtools.TestCase):

    def test_basic(self):
        sot = project.Project()
        self.assertEqual('project', sot.resource_key)
        self.assertEqual('projects', sot.resources_key)
        self.assertEqual('/projects', sot.base_path)
        self.assertEqual('identity', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_retrieve)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = project.Project(EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
        self.assertTrue(sot.is_enabled)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)



# -*- coding: utf-8 -*-

from ecl.virtual_network_appliance.v1 \
    import virtual_network_appliance as _virtual_network_appliance
from ecl.virtual_network_appliance.v1 \
    import virtual_network_appliance_plan as _virtual_network_appliance_plan
from ecl.virtual_network_appliance.v1 import operation as _operation
from ecl import exceptions
from ecl import proxy2


class Proxy(proxy2.BaseProxy):

    @proxy2._check_resource(strict=False)
    def _get(self, resource_type, value=None, requires_id=True, **params):
        """Get a resource

        :param resource_type: The type of resource to get.
        :type resource_type: :class:`~ecl.resource2.Resource`
        :param value: The value to get. Can be either the ID of a
                      resource or a :class:`~ecl.resource2.Resource`
                      subclass.
        :param dict params: Parameters to be passed onto the
                            :meth:`~ecl.resource2.Resource.get` method.
                            These are used as query parameters
                            with GET request.
        :returns: The result of the ``get``
        :rtype: :class:`~ecl.resource2.Resource`
        """
        res = self._get_resource(resource_type, value)
        if not res.allow_get:
            raise exceptions.MethodNotSupported(res, "get")

        try:
            request = res._prepare_request(requires_id=requires_id)
            query_params = res._query_mapping._transpose(params)
            response = self.session.get(request.uri,
                                        endpoint_filter=res.service,
                                        params=query_params)
            res._translate_response(response)
            return res
        except exceptions.NotFoundException as e:
            raise exceptions.ResourceNotFound(
                message="No %s found for %s" %
                        (resource_type.__name__, value),
                details=e.details, response=e.response,
                request_id=e.request_id, url=e.url, method=e.method,
                http_status=e.http_status, cause=e.cause)

    def virtual_network_appliance_plans(self, **params):
        """List virtual network appliance plans.

        :param params: The parameters as query string format
            to get network appliance plans.
        :returns: A list of network appliance plans.
        :rtype: list of :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance_plan.VirtualNetworkAppliancePlan`
        """
        return list(self._list(
            _virtual_network_appliance_plan.VirtualNetworkAppliancePlan,
            paginated=False, **params))

    def get_virtual_network_appliance_plan(
            self, virtual_network_appliance_plan_id, **params):
        """Show virtual network appliance plan.

        :param string virtual_network_appliance_plan_id:
            ID of specified virtual network appliance plan.
        :param params: The parameters as query string format
            to get a network appliance plan.
        :return: :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance_plan.VirtualNetworkAppliancePlan`
        """
        return self._get(
            _virtual_network_appliance_plan.VirtualNetworkAppliancePlan,
            virtual_network_appliance_plan_id,
            **params
        )

    def virtual_network_appliances(self, **params):
        """List virtual network appliances.

        :param params: The parameters as query string format
            to get list of network appliance.
        :returns: A list of network appliance.
        :rtype: list of :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance.VirtualNetworkAppliance`
        """
        return list(self._list(
            _virtual_network_appliance.VirtualNetworkAppliance,
            paginated=False, **params))

    def get_virtual_network_appliance(self, virtual_network_appliance_id):
        """Show virtual network appliance.

        :param string virtual_network_appliance_id:
            ID of specified virtual network appliance.
        :return: :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance.VirtualNetworkAppliance`
        """
        return self._get(_virtual_network_appliance.VirtualNetworkAppliance,
                         virtual_network_appliance_id)

    def update_virtual_network_appliance(self, virtual_network_appliance,
                                         **body):
        """Update a virtual network appliance.

        :param virtual_network_appliance:
            ID of specified virtual network appliance.
        :param :attrs \*\*params: Parameters for updating
            specified virtual network appliance.
        :returns: :class:`~ecl.virtual_network_appliance.
            v1.virtual_network_appliance.VirtualNetworkAppliance`
        """
        return self._update(_virtual_network_appliance.VirtualNetworkAppliance,
                            virtual_network_appliance, **body)

    def find_virtual_network_appliance(self, name_or_id, ignore_missing=False):
        """Find a single virtual network appliance.

        :param name_or_id: The name or ID of a virtual network appliance.
        :param bool ignore_missing: When set to ``False``
            :class:`~ecl.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance.VirtualNetworkAppliance` or None
        """
        return self._find(_virtual_network_appliance.VirtualNetworkAppliance,
                          name_or_id,
                          ignore_missing=ignore_missing)

    def delete_virtual_network_appliance(self, virtual_network_appliance_id,
                                         ignore_missing=False):
        """Delete virtual network appliance.

        :param virtual_network_appliance_id:
            The ID of a virtual network appliance.
        :param bool ignore_missing: When set to ``False``
            :class:`~ecl.exceptions.ResourceNotFound` will be
            raised when the port does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent port.
        :returns: ``None``
        """
        self._delete(_virtual_network_appliance.VirtualNetworkAppliance,
                     virtual_network_appliance_id,
                     ignore_missing=ignore_missing)

    def create_virtual_network_appliance(
            self,
            virtual_network_appliance_plan_id,
            interfaces,
            name=None,
            description=None,
            tags=None,
            default_gateway=None,
            availability_zone=None):
        """Create virtual network appliance.

        :param string virtual_network_appliance_plan_id:
            Plan ID for virtual network appliance.
        :param dict interfaces: Interface definition dictionary.
        :param string name: Name of virtual network appliance.
        :param string description: Description of virtual network appliance.
        :param dict tags: Tags of virtual network appliance.
        :param string default_gateway: Default gateway address for
            virtual network appliance.
        :param string availability_zone: Availability Zone
            for virtual network appliance.
        :returns: :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance.VirtualNetworkAppliance`
        """
        body = {}
        body["virtual_network_appliance_plan_id"] = \
            virtual_network_appliance_plan_id
        body["interfaces"] = interfaces
        if name:
            body["name"] = name
        if description:
            body["description"] = description
        if tags:
            body["tags"] = tags
        if default_gateway:
            body["default_gateway"] = default_gateway
        if availability_zone:
            body["availability_zone"] = availability_zone
        return self._create(_virtual_network_appliance.VirtualNetworkAppliance,
                            **body)

    def start_virtual_network_appliance(self, virtual_network_appliance):
        """Start the virtual network appliance.

        :param virtual_network_appliance:
            The ID of a virtual network appliance.
        :return: <Response 200>
        """
        virtual_network_appliance = \
            self.get_virtual_network_appliance(virtual_network_appliance)
        return virtual_network_appliance.start(self.session)

    def get_virtual_network_appliance_console(self,
                                              virtual_network_appliance,
                                              vnc_type):
        """Get console information for the virtual network appliance.

        :param virtual_network_appliance: Either the ID of a server or a
            :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance.VirtualNetworkAppliance` instance.
        :param vnc_type: should be `~string "novnc"`
        :return: <Response 200>
        """
        virtual_network_appliance = \
            self.get_virtual_network_appliance(virtual_network_appliance)
        return virtual_network_appliance.get_console(self.session, vnc_type)

    def stop_virtual_network_appliance(self, virtual_network_appliance):
        """Stop the virtual network appliance.

        :param virtual_network_appliance:
            The ID of a virtual network appliance.
        :return: <Response 200>
        """
        virtual_network_appliance = \
            self.get_virtual_network_appliance(virtual_network_appliance)
        return virtual_network_appliance.stop(self.session)

    def restart_virtual_network_appliance(self, virtual_network_appliance):
        """Restart the virtual network appliance.

        :param virtual_network_appliance:
            The ID of a virtual network appliance.
        :return: <Response 200>
        """
        virtual_network_appliance = \
            self.get_virtual_network_appliance(virtual_network_appliance)
        return virtual_network_appliance.restart(self.session)

    def reset_password_virtual_network_appliance(self,
                                                 virtual_network_appliance):
        """Reset the password of virtual network appliance.

        :param virtual_network_appliance:
            The ID of a virtual network appliance.
        :return: <Response 200>
        """
        virtual_network_appliance = \
            self.get_virtual_network_appliance(virtual_network_appliance)
        return virtual_network_appliance.reset_password(self.session)

    def operations(self, resource_ids=[], no_deleted=True, latest=False):
        """List operations.

        :param resource_ids: The list of resouce(appliance) IDs.
            If user specify this parameter,
            operations that has resource_id attribute is included in
            this list will be returned.
        :param bool no_deleted:
            ``True`` You can get operations for only existing resources.
            ``False`` You can get operations for all resources even removed.
        :param bool latest:
            ``True`` You can get only the latest operations for each resource.
            ``False`` You can get all of operations for each resource.
        :returns: A list of operation objects
        :rtype: list of :class:`~ecl.virtual_network_appliance.v1.
            operation.Operation`
        """
        return list(self._list(_operation.Operation, paginated=False,
                               resource_ids=resource_ids,
                               no_deleted=no_deleted,
                               latest=latest))

    def get_operation(
            self, operation_id):
        """Show operation.

        :param string operation_id: ID of specified operation.
        :return: :class:`~ecl.virtual_network_appliance.v1.
            operation.Operation`
        """
        return self._get(_operation.Operation, operation_id)

    def find_virtual_network_appliance_plan(self, name_or_id,
                                            ignore_missing=False):
        """Find a single virtual network appliance plan.

        :param name_or_id: The name or ID of a virtual network appliance plan.
        :param bool ignore_missing: When set to ``False``
            :class:`~ecl.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.virtual_network_appliance.v1.
            virtual_network_appliance_plan.VirtualNetworkAppliancePlan`
            or None
        """
        return self._find(
            _virtual_network_appliance_plan.VirtualNetworkAppliancePlan,
            name_or_id,
            ignore_missing=ignore_missing)



# -*- coding: utf-8 -*-

from ecl.tests.functional import base


class TestIntenetService(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestIntenetService, cls).setUpClass()
        cls.one_inet_service = None
        inet_services = cls.conn.network.internet_services()
        if inet_services and len(inet_services) > 0:
            cls.one_inet_service = inet_services[0]

    def test_get(self):
        sot = self.conn.network.get_internet_service(self.one_inet_service.id)
        self.assertEqual(self.one_inet_service.id, sot.id)

    def test_find_by_id(self):
        sot = self.conn.network.find_internet_service(self.one_inet_service.id)
        self.assertEqual(self.one_inet_service.id, sot.id)

    def test_find_by_name(self):
        sot = self.conn.network.find_internet_service(self.one_inet_service.name)
        self.assertEqual(self.one_inet_service.id, sot.id)

class TestInternetGateway(base.BaseFunctionalTest):

    @classmethod
    def prepare_create(cls):
        inet_services = cls.conn.network.internet_services()
        if inet_services and len(inet_services) > 0:
            cls.one_inet_service = inet_services[0]
        qos_options = cls.conn.network.qos_options()
        for qos in qos_options:
            if qos.service_type == "internet" and qos.name.startswith("10Mbps"):
                cls.one_qos_option = qos
                break

    @classmethod
    def setUpClass(cls):
        super(TestInternetGateway, cls).setUpClass()
        cls.gw_name = "Test-sdk-inet-gw"
        cls.one_inet_gw = None
        cls.one_qos_option = None
        cls.one_inet_service = None

        inet_gws = cls.conn.network.internet_gateways()
        for inet_gw in inet_gws:
            if inet_gw.name == cls.gw_name:
                cls.one_inet_gw = inet_gw
                break
        if cls.one_inet_gw is None:
            cls.prepare_create()
            cls.one_inet_gw = cls.conn.network.create_internet_gateway(
                internet_service_id=cls.one_inet_service.id,
                qos_option_id=cls.one_qos_option.id,
                name=cls.gw_name,
            )
            cls.one_inet_gw .wait_for_gateway()

    def test_get(self):
        if self.one_inet_gw is not None:
            sot = self.conn.network.get_internet_gateway(self.one_inet_gw.id)
            self.assertEqual(self.one_inet_gw.id, sot.id)

    def _test_create(self):
        if self.one_inet_service is None:
            self.prepare_create()
        sot = self.conn.network.create_internet_gateway(
            internet_service_id=self.one_inet_service.id,
            qos_option_id=self.one_qos_option.id,
            name=self.gw_name,
        )
        sot.wait_for_gateway()
        if self.one_inet_gw is None:
            self.one_inet_gw = sot

    def test_update(self):
        sot = self.conn.network.update_internet_gateway(self.one_inet_gw.id,
                                                        description="(edited)")
        print sot

    def _test_delete(self):
        sot = self.conn.network.delete_internet_gateway(self.one_inet_gw.id)
        self.assertIsNone(sot)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.identity.v3 import credential as _credential
from ecl.identity.v3 import domain as _domain
from ecl.identity.v3 import endpoint as _endpoint
from ecl.identity.v3 import group as _group
from ecl.identity.v3 import policy as _policy
from ecl.identity.v3 import project as _project
from ecl.identity.v3 import region as _region
from ecl.identity.v3 import service as _service
from ecl.identity.v3 import trust as _trust
from ecl.identity.v3 import user as _user
from ecl import proxy


class Proxy(proxy.BaseProxy):

    def create_credential(self, **attrs):
        """Create a new credential from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~ecl.identity.v3.credential.Credential`,
            comprised of the properties on the Credential class.

        :returns: The results of credential creation
        :rtype: :class:`~ecl.identity.v3.credential.Credential`
        """
        return self._create(_credential.Credential, **attrs)

    def delete_credential(self, credential, ignore_missing=False):
        """Delete a credential

        :param credential: The value can be either the ID of a credential or a
               :class:`~ecl.identity.v3.credential.Credential` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the credential does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent credential.

        :returns: ``None``
        """
        self._delete(_credential.Credential, credential,
                     ignore_missing=ignore_missing)

    def find_credential(self, name_or_id, ignore_missing=False):
        """Find a single credential

        :param name_or_id: The name or ID of a credential.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.credential.Credential`
                  or None
        """
        return self._find(_credential.Credential, name_or_id,
                          ignore_missing=ignore_missing)

    def get_credential(self, credential):
        """Get a single credential

        :param credential: The value can be the ID of a credential or a
            :class:`~ecl.identity.v3.credential.Credential` instance.

        :returns: One :class:`~ecl.identity.v3.credential.Credential`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_credential.Credential, credential)

    def credentials(self, **query):
        """Retrieve a generator of credentials

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of credentials instances.
        :rtype: :class:`~ecl.identity.v3.credential.Credential`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_credential.Credential, paginated=False, **query)

    def update_credential(self, credential, **attrs):
        """Update a credential

        :param credential: Either the ID of a credential or a
            :class:`~ecl.identity.v3.credential.Credential` instance.
        :attrs kwargs: The attributes to update on the credential represented
                       by ``value``.

        :returns: The updated credential
        :rtype: :class:`~ecl.identity.v3.credential.Credential`
        """
        return self._update(_credential.Credential, credential, **attrs)

    def create_domain(self, **attrs):
        """Create a new domain from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.domain.Domain`,
                           comprised of the properties on the Domain class.

        :returns: The results of domain creation
        :rtype: :class:`~ecl.identity.v3.domain.Domain`
        """
        return self._create(_domain.Domain, **attrs)

    def delete_domain(self, domain, ignore_missing=False):
        """Delete a domain

        :param domain: The value can be either the ID of a domain or a
                       :class:`~ecl.identity.v3.domain.Domain` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the domain does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent domain.

        :returns: ``None``
        """
        self._delete(_domain.Domain, domain, ignore_missing=ignore_missing)

    def find_domain(self, name_or_id, ignore_missing=False):
        """Find a single domain

        :param name_or_id: The name or ID of a domain.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.domain.Domain` or None
        """
        return self._find(_domain.Domain, name_or_id,
                          ignore_missing=ignore_missing)

    def get_domain(self, domain):
        """Get a single domain

        :param domain: The value can be the ID of a domain or a
                       :class:`~ecl.identity.v3.domain.Domain` instance.

        :returns: One :class:`~ecl.identity.v3.domain.Domain`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_domain.Domain, domain)

    def domains(self, **query):
        """Retrieve a generator of domains

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of domain instances.
        :rtype: :class:`~ecl.identity.v3.domain.Domain`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_domain.Domain, paginated=False, **query)

    def update_domain(self, domain, **attrs):
        """Update a domain

        :param domain: Either the ID of a domain or a
                       :class:`~ecl.identity.v3.domain.Domain` instance.
        :attrs kwargs: The attributes to update on the domain represented
                       by ``value``.

        :returns: The updated domain
        :rtype: :class:`~ecl.identity.v3.domain.Domain`
        """
        return self._update(_domain.Domain, domain, **attrs)

    def create_endpoint(self, **attrs):
        """Create a new endpoint from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.endpoint.Endpoint`,
                           comprised of the properties on the Endpoint class.

        :returns: The results of endpoint creation
        :rtype: :class:`~ecl.identity.v3.endpoint.Endpoint`
        """
        return self._create(_endpoint.Endpoint, **attrs)

    def delete_endpoint(self, endpoint, ignore_missing=False):
        """Delete an endpoint

        :param endpoint: The value can be either the ID of an endpoint or a
               :class:`~ecl.identity.v3.endpoint.Endpoint` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the endpoint does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent endpoint.

        :returns: ``None``
        """
        self._delete(_endpoint.Endpoint, endpoint,
                     ignore_missing=ignore_missing)

    def find_endpoint(self, name_or_id, ignore_missing=False):
        """Find a single endpoint

        :param name_or_id: The name or ID of a endpoint.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.endpoint.Endpoint` or None
        """
        return self._find(_endpoint.Endpoint, name_or_id,
                          ignore_missing=ignore_missing)

    def get_endpoint(self, endpoint):
        """Get a single endpoint

        :param endpoint: The value can be the ID of an endpoint or a
                         :class:`~ecl.identity.v3.endpoint.Endpoint`
                         instance.

        :returns: One :class:`~ecl.identity.v3.endpoint.Endpoint`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_endpoint.Endpoint, endpoint)

    def endpoints(self, **query):
        """Retrieve a generator of endpoints

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of endpoint instances.
        :rtype: :class:`~ecl.identity.v3.endpoint.Endpoint`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_endpoint.Endpoint, paginated=False, **query)

    def update_endpoint(self, endpoint, **attrs):
        """Update a endpoint

        :param endpoint: Either the ID of a endpoint or a
                         :class:`~ecl.identity.v3.endpoint.Endpoint`
                         instance.
        :attrs kwargs: The attributes to update on the endpoint represented
                       by ``value``.

        :returns: The updated endpoint
        :rtype: :class:`~ecl.identity.v3.endpoint.Endpoint`
        """
        return self._update(_endpoint.Endpoint, endpoint, **attrs)

    def create_group(self, **attrs):
        """Create a new group from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.group.Group`,
                           comprised of the properties on the Group class.

        :returns: The results of group creation
        :rtype: :class:`~ecl.identity.v3.group.Group`
        """
        return self._create(_group.Group, **attrs)

    def delete_group(self, group, ignore_missing=False):
        """Delete a group

        :param group: The value can be either the ID of a group or a
                      :class:`~ecl.identity.v3.group.Group` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the group does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent group.

        :returns: ``None``
        """
        self._delete(_group.Group, group, ignore_missing=ignore_missing)

    def find_group(self, name_or_id, ignore_missing=False):
        """Find a single group

        :param name_or_id: The name or ID of a group.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.group.Group` or None
        """
        return self._find(_group.Group, name_or_id,
                          ignore_missing=ignore_missing)

    def get_group(self, group):
        """Get a single group

        :param group: The value can be the ID of a group or a
                      :class:`~ecl.identity.v3.group.Group`
                      instance.

        :returns: One :class:`~ecl.identity.v3.group.Group`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_group.Group, group)

    def groups(self, **query):
        """Retrieve a generator of groups

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of group instances.
        :rtype: :class:`~ecl.identity.v3.group.Group`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_group.Group, paginated=False, **query)

    def update_group(self, group, **attrs):
        """Update a group

        :param group: Either the ID of a group or a
                      :class:`~ecl.identity.v3.group.Group` instance.
        :attrs kwargs: The attributes to update on the group represented
                       by ``value``.

        :returns: The updated group
        :rtype: :class:`~ecl.identity.v3.group.Group`
        """
        return self._update(_group.Group, group, **attrs)

    def create_policy(self, **attrs):
        """Create a new policy from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.policy.Policy`,
                           comprised of the properties on the Policy class.

        :returns: The results of policy creation
        :rtype: :class:`~ecl.identity.v3.policy.Policy`
        """
        return self._create(_policy.Policy, **attrs)

    def delete_policy(self, policy, ignore_missing=False):
        """Delete a policy

        :param policy: The value can be either the ID of a policy or a
                       :class:`~ecl.identity.v3.policy.Policy` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the policy does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent policy.

        :returns: ``None``
        """
        self._delete(_policy.Policy, policy, ignore_missing=ignore_missing)

    def find_policy(self, name_or_id, ignore_missing=False):
        """Find a single policy

        :param name_or_id: The name or ID of a policy.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.policy.Policy` or None
        """
        return self._find(_policy.Policy, name_or_id,
                          ignore_missing=ignore_missing)

    def get_policy(self, policy):
        """Get a single policy

        :param policy: The value can be the ID of a policy or a
                       :class:`~ecl.identity.v3.policy.Policy` instance.

        :returns: One :class:`~ecl.identity.v3.policy.Policy`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_policy.Policy, policy)

    def policies(self, **query):
        """Retrieve a generator of policies

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of policy instances.
        :rtype: :class:`~ecl.identity.v3.policy.Policy`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_policy.Policy, paginated=False, **query)

    def update_policy(self, policy, **attrs):
        """Update a policy

        :param policy: Either the ID of a policy or a
                       :class:`~ecl.identity.v3.policy.Policy` instance.
        :attrs kwargs: The attributes to update on the policy represented
                       by ``value``.

        :returns: The updated policy
        :rtype: :class:`~ecl.identity.v3.policy.Policy`
        """
        return self._update(_policy.Policy, policy, **attrs)

    def create_project(self, **attrs):
        """Create a new project from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.project.Project`,
                           comprised of the properties on the Project class.

        :returns: The results of project creation
        :rtype: :class:`~ecl.identity.v3.project.Project`
        """
        return self._create(_project.Project, **attrs)

    def delete_project(self, project, ignore_missing=False):
        """Delete a project

        :param project: The value can be either the ID of a project or a
            :class:`~ecl.identity.v3.project.Project` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the project does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent project.

        :returns: ``None``
        """
        self._delete(_project.Project, project, ignore_missing=ignore_missing)

    def find_project(self, name_or_id, ignore_missing=False):
        """Find a single project

        :param name_or_id: The name or ID of a project.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.project.Project` or None
        """
        return self._find(_project.Project, name_or_id,
                          ignore_missing=ignore_missing)

    def get_project(self, project):
        """Get a single project

        :param project: The value can be the ID of a project or a
            :class:`~ecl.identity.v3.project.Project` instance.

        :returns: One :class:`~ecl.identity.v3.project.Project`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_project.Project, project)

    def projects(self, **query):
        """Retrieve a generator of projects

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of project instances.
        :rtype: :class:`~ecl.identity.v3.project.Project`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_project.Project, paginated=False, **query)

    def update_project(self, project, **attrs):
        """Update a project

        :param project: Either the ID of a project or a
            :class:`~ecl.identity.v3.project.Project` instance.
        :attrs kwargs: The attributes to update on the project represented
                       by ``value``.

        :returns: The updated project
        :rtype: :class:`~ecl.identity.v3.project.Project`
        """
        return self._update(_project.Project, project, **attrs)

    def create_service(self, **attrs):
        """Create a new service from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.service.Service`,
                           comprised of the properties on the Service class.

        :returns: The results of service creation
        :rtype: :class:`~ecl.identity.v3.service.Service`
        """
        return self._create(_service.Service, **attrs)

    def delete_service(self, service, ignore_missing=False):
        """Delete a service

        :param service: The value can be either the ID of a service or a
            :class:`~ecl.identity.v3.service.Service` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the service does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent service.

        :returns: ``None``
        """
        self._delete(_service.Service, service, ignore_missing=ignore_missing)

    def find_service(self, name_or_id, ignore_missing=False):
        """Find a single service

        :param name_or_id: The name or ID of a service.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.service.Service` or None
        """
        return self._find(_service.Service, name_or_id,
                          ignore_missing=ignore_missing)

    def get_service(self, service):
        """Get a single service

        :param service: The value can be the ID of a service or a
            :class:`~ecl.identity.v3.service.Service` instance.

        :returns: One :class:`~ecl.identity.v3.service.Service`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_service.Service, service)

    def services(self, **query):
        """Retrieve a generator of services

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of service instances.
        :rtype: :class:`~ecl.identity.v3.service.Service`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_service.Service, paginated=False, **query)

    def update_service(self, service, **attrs):
        """Update a service

        :param service: Either the ID of a service or a
            :class:`~ecl.identity.v3.service.Service` instance.
        :attrs kwargs: The attributes to update on the service represented
                       by ``value``.

        :returns: The updated service
        :rtype: :class:`~ecl.identity.v3.service.Service`
        """
        return self._update(_service.Service, service, **attrs)

    def create_user(self, **attrs):
        """Create a new user from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.user.User`,
                           comprised of the properties on the User class.

        :returns: The results of user creation
        :rtype: :class:`~ecl.identity.v3.user.User`
        """
        return self._create(_user.User, **attrs)

    def delete_user(self, user, ignore_missing=False):
        """Delete a user

        :param user: The value can be either the ID of a user or a
                     :class:`~ecl.identity.v3.user.User` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the user does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent user.

        :returns: ``None``
        """
        self._delete(_user.User, user, ignore_missing=ignore_missing)

    def find_user(self, name_or_id, ignore_missing=False):
        """Find a single user

        :param name_or_id: The name or ID of a user.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.user.User` or None
        """
        return self._find(_user.User, name_or_id,
                          ignore_missing=ignore_missing)

    def get_user(self, user):
        """Get a single user

        :param user: The value can be the ID of a user or a
                     :class:`~ecl.identity.v3.user.User` instance.

        :returns: One :class:`~ecl.identity.v3.user.User`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_user.User, user)

    def users(self, **query):
        """Retrieve a generator of users

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of user instances.
        :rtype: :class:`~ecl.identity.v3.user.User`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_user.User, paginated=False, **query)

    def update_user(self, user, **attrs):
        """Update a user

        :param user: Either the ID of a user or a
                     :class:`~ecl.identity.v3.user.User` instance.
        :attrs kwargs: The attributes to update on the user represented
                       by ``value``.

        :returns: The updated user
        :rtype: :class:`~ecl.identity.v3.user.User`
        """
        return self._update(_user.User, user, **attrs)

    def create_trust(self, **attrs):
        """Create a new trust from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.trust.Trust`,
                           comprised of the properties on the Trust class.

        :returns: The results of trust creation
        :rtype: :class:`~ecl.identity.v3.trust.Trust`
        """
        return self._create(_trust.Trust, **attrs)

    def delete_trust(self, trust, ignore_missing=False):
        """Delete a trust

        :param trust: The value can be either the ID of a trust or a
               :class:`~ecl.identity.v3.trust.Trust` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the credential does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent credential.

        :returns: ``None``
        """
        self._delete(_trust.Trust, trust, ignore_missing=ignore_missing)

    def find_trust(self, name_or_id, ignore_missing=False):
        """Find a single trust

        :param name_or_id: The name or ID of a trust.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.identity.v3.trust.Trust` or None
        """
        return self._find(_trust.Trust, name_or_id,
                          ignore_missing=ignore_missing)

    def get_trust(self, trust):
        """Get a single trust

        :param trust: The value can be the ID of a trust or a
                      :class:`~ecl.identity.v3.trust.Trust` instance.

        :returns: One :class:`~ecl.identity.v3.trust.Trust`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_trust.Trust, trust)

    def trusts(self, **query):
        """Retrieve a generator of trusts

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of trust instances.
        :rtype: :class:`~ecl.identity.v3.trust.Trust`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_trust.Trust, paginated=False, **query)

    def create_region(self, **attrs):
        """Create a new region from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.identity.v3.region.Region`,
                           comprised of the properties on the Region class.

        :returns: The results of region creation.
        :rtype: :class:`~ecl.identity.v3.region.Region`
        """
        return self._create(_region.Region, **attrs)

    def delete_region(self, region, ignore_missing=False):
        """Delete a region

        :param region: The value can be either the ID of a region or a
               :class:`~ecl.identity.v3.region.Region` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the region does not exist.
                    When set to ``True``, no exception will be thrown when
                    attempting to delete a nonexistent region.

        :returns: ``None``
        """
        self._delete(_region.Region, region, ignore_missing=ignore_missing)

    def find_region(self, name_or_id, ignore_missing=False):
        """Find a single region

        :param name_or_id: The name or ID of a region.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the region does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent region.
        :returns: One :class:`~ecl.identity.v3.region.Region` or None
        """
        return self._find(_region.Region, name_or_id,
                          ignore_missing=ignore_missing)

    def get_region(self, region):
        """Get a single region

        :param region: The value can be the ID of a region or a
                       :class:`~ecl.identity.v3.region.Region` instance.

        :returns: One :class:`~ecl.identity.v3.region.Region`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no matching region can be found.
        """
        return self._get(_region.Region, region)

    def regions(self, **query):
        """Retrieve a generator of regions

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the regions being returned.

        :returns: A generator of region instances.
        :rtype: :class:`~ecl.identity.v3.region.Region`
        """
        # TODO(briancurtin): This is paginated but requires base list changes.
        return self._list(_region.Region, paginated=False, **query)

    def update_region(self, region, **attrs):
        """Update a region

        :param region: Either the ID of a region or a
                      :class:`~ecl.identity.v3.region.Region` instance.
        :attrs kwargs: The attributes to update on the region represented
                       by ``value``.

        :returns: The updated region.
        :rtype: :class:`~ecl.identity.v3.region.Region`
        """
        return self._update(_region.Region, region, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import  testtools

from ecl.sss.v1 import role

BASIC_EXAMPLE = {
  "user_id":"ecid1000024459",
  "region_id":"jp1",
  "region_name":"jp1",
  "tenant_id":"3c90482157664923ada457950a96aca8",
  "tenant_name":"Cheese",
  "role_name":"_member_"
}

class TestRole(testtools.TestCase):

    def test_role(self):
        sot = role.Role()
        self.assertEqual(None, sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("/roles", sot.base_path)
        self.assertEqual('sss', sot.service.service_type)

        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.put_create)

    def test_make_role(self):
        sot = role.Role(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['user_id'], sot.user_id)
        self.assertEqual(BASIC_EXAMPLE['region_id'], sot.region_id)
        self.assertEqual(BASIC_EXAMPLE['region_name'], sot.region_name)
        self.assertEqual(BASIC_EXAMPLE['tenant_id'], sot.tenant_id)
        self.assertEqual(BASIC_EXAMPLE['tenant_name'], sot.tenant_name)
        self.assertEqual(BASIC_EXAMPLE['role_name'], sot.role_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.network.v2 import quota

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "colocation_logical_link": 2,
    "common_function_gateway": 1,
    "firewall": 2,
    "id": IDENTIFIER,
    "interdc_gateway": 1,
    "internet_gateway": 1,
    "load_balancer": 2,
    "network": 2,
    "port": 30,
    "subnet": 5,
    "tenant_id": IDENTIFIER,
    "vpn_gateway": 1
}



class TestQuota(testtools.TestCase):

    def test_basic(self):
        sot = quota.Quota()
        self.assertEqual('quota', sot.resource_key)
        self.assertEqual('quotas', sot.resources_key)
        self.assertEqual('/quotas', sot.base_path)
        self.assertEqual('network', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = quota.Quota(**EXAMPLE)
        self.assertEqual(EXAMPLE['colocation_logical_link'], sot.colocation_logical_link)
        self.assertEqual(EXAMPLE['common_function_gateway'], sot.common_function_gateway)
        self.assertEqual(EXAMPLE['firewall'], sot.firewall)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['interdc_gateway'], sot.interdc_gateway)
        self.assertEqual(EXAMPLE['interdc_gateway'], sot.internet_gateway)
        self.assertEqual(EXAMPLE['load_balancer'], sot.load_balancer)
        self.assertEqual(EXAMPLE['network'], sot.networks)
        self.assertEqual(EXAMPLE['port'], sot.ports)
        self.assertEqual(EXAMPLE['subnet'], sot.subnets)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
        self.assertEqual(EXAMPLE['vpn_gateway'], sot.vpn_gateway)

class TestQuotaDefault(testtools.TestCase):

    def test_basic(self):
        default = quota.QuotaDefault()
        self.assertEqual('quota', default.resource_key)
        self.assertEqual('quotas', default.resources_key)
        self.assertEqual('/quotas/%(project)s/default', default.base_path)
        self.assertEqual('network', default.service.service_type)
        self.assertFalse(default.allow_create)
        self.assertTrue(default.allow_get)
        self.assertFalse(default.allow_update)
        self.assertFalse(default.allow_delete)
        self.assertFalse(default.allow_list)

    def test_make_it(self):
        default = quota.QuotaDefault(**EXAMPLE)
        self.assertEqual(EXAMPLE['colocation_logical_link'], default.colocation_logical_link)
        self.assertEqual(EXAMPLE['common_function_gateway'], default.common_function_gateway)
        self.assertEqual(EXAMPLE['firewall'], default.firewall)
        self.assertEqual(EXAMPLE['id'], default.id)
        self.assertEqual(EXAMPLE['interdc_gateway'], default.interdc_gateway)
        self.assertEqual(EXAMPLE['interdc_gateway'], default.internet_gateway)
        self.assertEqual(EXAMPLE['load_balancer'], default.load_balancer)
        self.assertEqual(EXAMPLE['network'], default.networks)
        self.assertEqual(EXAMPLE['port'], default.ports)
        self.assertEqual(EXAMPLE['subnet'], default.subnets)
        self.assertEqual(EXAMPLE['tenant_id'], default.project_id)
        self.assertEqual(EXAMPLE['vpn_gateway'], default.vpn_gateway)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
import time
from ecl.tests.functional import base

class TestFirewall(base.BaseFunctionalTest):

    def test_01_list_firewall(self):
        firewalls = list(self.conn.network.firewalls(
            status="ACTIVE"))
        print "Total: ", len(firewalls)
        print firewalls

        for firewall in firewalls:
            print firewall.name, " ", firewall.id

        firewall = firewalls[0]
        self.assertIsInstance(firewall.admin_username, six.string_types)
        #self.assertIsInstance(firewall.availability_zone, six.string_types)
        #self.assertIsInstance(firewall.default_gateway, six.string_types)
        self.assertIsInstance(firewall.description, six.string_types)
        self.assertIsInstance(firewall.firewall_plan_id, six.string_types)
        self.assertIsInstance(firewall.id, six.string_types)
        self.assertIsInstance(firewall.name, six.string_types)
        self.assertIsInstance(firewall.status, six.string_types)
        self.assertIsInstance(firewall.tenant_id, six.string_types)
        self.assertIsInstance(firewall.user_username, six.string_types)

    @classmethod
    def test_02_create_firewall(cls):
        firewall = cls.conn.network.create_firewall(
            firewall_plan_id="4441399b-a3dd-40c0-903f-a02e685f7e8c",
        )
        cls.id = firewall.id
        time.sleep(360)

    def test_03_show_firewalll(self):
        firewall = self.conn.network.show_firewall(
            self.id
        )
        print firewall
        self.assertIsInstance(firewall.interfaces, list)

    def test_04_update_firewall(self):
        firewall = self.conn.network.update_firewall(
            self.id,
            description="updated description",
            name="sdk_firewall"
        )
        print firewall

    def test_05_delete_firewall(self):
        firewall = self.conn.network.delete_firewall(
            self.id
        )

    def test_06_list_firewall(self):
        firewalls = list(self.conn.network.firewalls(
            status="ACTIVE"))
        print "Total: ", len(firewalls)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.compute import compute_service
from ecl.compute.v2 import metadata as _metadata
from ecl import exceptions
from ecl import resource2
from ecl import utils


class Server(resource2.Resource, _metadata.MetadataMixin):
    resource_key = 'server'
    resources_key = 'servers'
    base_path = '/servers'
    service = compute_service.ComputeService()

    # capabilities
    allow_create = True
    allow_get = True
    allow_update = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource2.QueryParameters("image", "flavor", "name",
                                               "status", "host",
                                               changes_since="changes-since")

    #: IPv4 Address
    access_ipv4 = resource2.Body('accessIPv4')
    accessIPv4 = resource2.Body('accessIPv4')
    #: IPv6 Address
    access_ipv6 = resource2.Body('accessIPv6')
    accessIPv6 = resource2.Body('accessIPv6')
    #: A dictionary of addresses this server can be accessed through.
    #: The dictionary contains keys such as ``private`` and ``public``,
    #: each containing a list of dictionaries for addresses of that type.
    #: The addresses are contained in a dictionary with keys ``addr``
    #: and ``version``, which is either 4 or 6 depending on the protocol
    #: of the IP address. *Type: dict*
    addresses = resource2.Body('addresses', type=dict)
    #: config drive
    config_drive = resource2.Body('config_drive')
    #: Timestamp of when the server was created.
    created_at = resource2.Body('created')
    #: The flavor reference, as a ID or full URL, for the flavor to use for
    #: this server.
    flavor_id = resource2.Body('flavorRef')
    #: The flavor property as returned from server.
    flavor = resource2.Body('flavor', type=dict)
    #: An ID representing the host of this server.
    host_id = resource2.Body('hostId')
    #: The image reference, as a ID or full URL, for the image to use for
    #: this server.
    image_id = resource2.Body('imageRef')
    #: The image property as returned from server.
    image = resource2.Body('image', type=dict)
    #: ID of the server
    id = resource2.Body('id')
    #: The name of an associated keypair
    key_name = resource2.Body('key_name')
    #: A list of dictionaries holding links relevant to this server.
    links = resource2.Body('links')
    #: Metadata stored for this server. *Type: dict*
    metadata = resource2.Body('metadata', type=dict)
    #: Name of the server
    name = resource2.Body('name')
    #: A networks object. Required parameter when there are multiple
    #: networks defined for the tenant. When you do not specify the
    #: networks parameter, the server attaches to the only network
    #: created for the current tenant.
    networks = resource2.Body('networks', type=dict)
    #: The disk configuration. Either AUTO or MANUAL.
    disk_config = resource2.Body('OS-DCF:diskConfig')
    diskConfig = resource2.Body('OS-DCF:diskConfig')
    #: The name of the availability zone this server is a part of.
    availability_zone = resource2.Body('OS-EXT-AZ:availability_zone')
    #: The power state of this server.
    power_state = resource2.Body('OS-EXT-STS:power_state')
    #: The task state of this server.
    task_state = resource2.Body('OS-EXT-STS:task_state')
    #: The VM state of this server.
    vm_state = resource2.Body('OS-EXT-STS:vm_state')
    #: A list of an attached volumes. Each item in the list contains at least
    #: an "id" key to identify the specific volumes.
    attached_volumes = resource2.Body(
        'os-extended-volumes:volumes_attached')
    #: The timestamp when the server was launched.
    launched_at = resource2.Body('OS-SRV-USG:launched_at')
    #: The timestamp when the server was terminated (if it has been).
    terminated_at = resource2.Body('OS-SRV-USG:terminated_at')
    #: of completion. Once it is completed, it will be 100.  *Type: int*
    progress = resource2.Body('progress', type=int)
    #: A list of applicable security groups. Each group contains keys for
    #: description, name, id, and rules.
    security_groups = resource2.Body('security_groups')
    #: The state this server is in. Valid values include ``ACTIVE``,
    #: ``BUILDING``, ``DELETED``, ``ERROR``, ``HARD_REBOOT``, ``PASSWORD``,
    #: ``PAUSED``, ``REBOOT``, ``REBUILD``, ``RESCUED``, ``RESIZED``,
    #: ``REVERT_RESIZE``, ``SHUTOFF``, ``SOFT_DELETED``, ``STOPPED``,
    #: ``SUSPENDED``, ``UNKNOWN``, or ``VERIFY_RESIZE``.
    status = resource2.Body('status')
    #: The ID of the project this server is associated with.
    project_id = resource2.Body('tenant_id')
    #: Timestamp of when this server was last updated.
    updated_at = resource2.Body('updated')
    #: The ID of the owners of this server.
    user_id = resource2.Body('user_id')
    #: The dict of tags of servers
    tags = resource2.Body('tags')
    #: The file path and contents, text only, to inject into the server at
    #: launch. The maximum size of the file path data is 255 bytes.
    #: The maximum limit is The number of allowed bytes in the decoded,
    #: rather than encoded, data.
    personality = resource2.Body('personality')
    #: Configuration information or scripts to use upon launch.
    #: Must be Base64 encoded.
    user_data = resource2.Body('user_data')
    #: Enables fine grained control of the block device mapping for an
    #: instance. This is typically used for booting servers from volumes.
    block_device_mapping = resource2.Body('block_device_mapping', type=dict)
    block_device_mapping_v2 = resource2.Body('block_device_mapping_v2', type=dict)
    #: admin user's password for this server
    admin_pass = resource2.Body('adminPass')
    adminPass = resource2.Body('adminPass')
    #: The dictionary of data to send to the scheduler.
    scheduler_hints = resource2.Body('os:scheduler_hints')
    #: Requested minim count of instance
    min_count = resource2.Body('min_count')
    #: Requested maximum count of instance
    max_count = resource2.Body('max_count')

    def create(self, session, prepend_key=True):
        """Create a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param prepend_key: A boolean indicating whether the resource_key
                            should be prepended in a resource creation
                            request. Default to True.

        :return: This :class:`Resource` instance.
        :raises: :exc:`~ecl.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_create` is not set to ``True``.
        """
        request = self._prepare_request(requires_id=False,
                                        prepend_key=prepend_key)

        az = request.body['server'].get('OS-EXT-AZ:availability_zone')
        if 'OS-EXT-AZ:availability_zone' in request.body['server']:
            del request.body['server']['OS-EXT-AZ:availability_zone']
        if az is not None:
            request.body['server'].update({'availability_zone': az})

        response = session.post(request.uri, endpoint_filter=self.service,
                                json=request.body, headers=request.headers)
        self._translate_response(response)
        return self

    def _action(self, session, body):
        """Preform server actions given the message body."""
        # NOTE: This is using Server.base_path instead of self.base_path
        # as both Server and ServerDetail instances can be acted on, but
        # the URL used is sans any additional /detail/ part.
        url = utils.urljoin(Server.base_path, self.id, 'action')
        headers = {'Accept': ''}
        return session.post(
            url, endpoint_filter=self.service, json=body, headers=headers)

    def create_image(self, session, name, metadata=None):
        """Create image from server.
        :param session
        :param name
        :param metadata
        """
        action = {'name': name}
        if metadata is not None:
            action['metadata'] = metadata
        body = {'createImage': action}
        return self._action(session, body)

    def get_console(self, session, vnc_type):
        action = {'type': vnc_type}
        body = {'os-getVNCConsole': action}
        resp = self._action(session, body)
        if resp:
            console_dict = resp.json()
            if console_dict:
                return console_dict.get("console")

    def start(self, session):
        body = {"os-start": None}
        return self._action(session, body)

    def stop(self, session):
        body = {"os-stop": None}
        return self._action(session, body)

    def resize(self, session, flavor):
        """Resize server to flavor reference."""
        body = {
            'resize': {
                'flavorRef': flavor,
                'OS-DCF:diskConfig': 'AUTO'
            }
        }
        self._action(session, body)

    def change_password(self, session, new_password):
        """Change the administrator password to the given password."""
        body = {'changePassword': {'adminPass': new_password}}
        self._action(session, body)

    def reboot(self, session, reboot_type):
        """Reboot server where reboot_type might be 'SOFT' or 'HARD'."""
        body = {'reboot': {'type': reboot_type}}
        self._action(session, body)

    def force_delete(self, session):
        """Force delete a server."""
        body = {'forceDelete': None}
        self._action(session, body)

    def rebuild(self, session, name, admin_password,
                preserve_ephemeral=False, image=None,
                access_ipv4=None, access_ipv6=None,
                metadata=None, personality=None):
        """Rebuild the server with the given arguments."""
        action = {
            'name': name,
            'adminPass': admin_password,
            'preserve_ephemeral': preserve_ephemeral
        }
        if image is not None:
            action['imageRef'] = resource2.Resource._get_id(image)
        if access_ipv4 is not None:
            action['accessIPv4'] = access_ipv4
        if access_ipv6 is not None:
            action['accessIPv6'] = access_ipv6
        if metadata is not None:
            action['metadata'] = metadata
        if personality is not None:
            action['personality'] = personality

        body = {'rebuild': action}
        response = self._action(session, body)
        self._translate_response(response)
        return self

    def confirm_resize(self, session):
        """Confirm the resize of the server."""
        body = {'confirmResize': None}
        self._action(session, body)

    def revert_resize(self, session):
        """Revert the resize of the server."""
        body = {'revertResize': None}
        self._action(session, body)

    def add_security_group(self, session, security_group):
        body = {"addSecurityGroup": {"name": security_group}}
        self._action(session, body)

    def remove_security_group(self, session, security_group):
        body = {"removeSecurityGroup": {"name": security_group}}
        self._action(session, body)

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))


class ServerDetail(Server):
    base_path = '/servers/detail'

    # capabilities
    allow_create = False
    allow_get = False
    allow_update = False
    allow_delete = False
    allow_list = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.dedicated_hypervisor import dedicated_hypervisor_service
from ecl import resource2


class Server(resource2.Resource):
    resources_key = "servers"
    resource_key = "server"
    base_path = '/servers'
    service = dedicated_hypervisor_service.DedicatedHypervisorService()

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_delete = True

    _query_mapping = resource2.QueryParameters("image", "flavor", "name",
                                               "status",
                                               changes_since="changes-since")

    # Properties
    #: id of Dedicated Hypervisor instance.
    id = resource2.Body('id')
    #: Link of the Dedicated Hypervisor server.
    links = resource2.Body('links')
    #: name of Dedicated Hypervisor instance.
    name = resource2.Body('name')
    #: description of Dedicated Hypervisor instance.
    description = resource2.Body('description')
    #: type of hypervisor. vsphere_esxi or hyper_v.
    hypervisor_type = resource2.Body('hypervisor_type')
    #: image id of Dedicated Hypervisor instance.
    imageRef = resource2.Body('imageRef')
    image = resource2.Body('imageRef')
    #: flavor id of Dedicated Hypervisor instance.
    flavorRef = resource2.Body('flavorRef')
    flavor = resource2.Body('flavorRef')
    #: Networks.
    networks = resource2.Body('networks')
    #: Password for the administrator.
    adminPass = resource2.Body('adminPass')
    admin_pass = resource2.Body('adminPass')
    #: Status of the Dedicated Hypervisor instance.
    status = resource2.Body('status')
    #: detail of baremetal server.
    baremetal_server = resource2.Body('baremetal_server')
    #: Server's availability zone.
    availability_zone = resource2.Body('availability_zone')


class ServerDetail(Server):
    base_path = '/servers/detail'

    # capabilities
    allow_list = True
    allow_get = False
    allow_create = False
    allow_delete = False


class ServerAction(Server):
    resource_key = None
    resources_key = None
    base_path = '/servers/%s/action'

    # capabilities
    allow_list = False
    allow_get = True
    allow_create = True
    allow_delete = False

    vm_id = resource2.Body('vm_id')
    vm_name = resource2.Body('vm_name')
    license_types = resource2.Body('license_types')
    job_id = resource2.Body('job_id')
    status = resource2.Body('status')
    message = resource2.Body('message')
    requested_param = resource2.Body('requested_param')

    def add_license(self, session, server_id, license_types, vm_id=None, vm_name=None):
        uri = self.base_path % server_id
        params = {}
        params["license_types"] = license_types
        if vm_id:
            params["vm_id"] = vm_id
        if vm_name:
            params["vm_name"] = vm_name
        body = { "add-license-to-vm": params }
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body
        )
        self._translate_response(resp, has_body=True)
        return self

    def get_add_license_job(self, session, server_id, job_id):
        uri = self.base_path % server_id
        body = { "get-result-for-add-license-to-vm": { "job_id": job_id } }
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body
        )
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.orchestration.v1 import resource


FAKE_ID = '32e39358-2422-4ad0-a1b5-dd60696bf564'
FAKE_NAME = 'test_stack'
FAKE = {
    'links': [{
        'href': 'http://res_link',
        'rel': 'self'
    }, {
        'href': 'http://stack_link',
        'rel': 'stack'
    }],
    'logical_resource_id': 'the_resource',
    'name': 'the_resource',
    'physical_resource_id': '9f38ab5a-37c8-4e40-9702-ce27fc5f6954',
    'required_by': [],
    'resource_type': 'OS::Heat::FakeResource',
    'status': 'CREATE_COMPLETE',
    'status_reason': 'state changed',
    'updated_time': '2015-03-09T12:15:57.233772',
}


class TestResource(testtools.TestCase):

    def test_basic(self):
        sot = resource.Resource()
        self.assertEqual('resource', sot.resource_key)
        self.assertEqual('resources', sot.resources_key)
        self.assertEqual('/stacks/%(stack_name)s/%(stack_id)s/resources',
                         sot.base_path)
        self.assertEqual('orchestration', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_retrieve)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = resource.Resource(**FAKE)
        self.assertEqual(FAKE['links'], sot.links)
        self.assertEqual(FAKE['logical_resource_id'], sot.logical_resource_id)
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['physical_resource_id'],
                         sot.physical_resource_id)
        self.assertEqual(FAKE['required_by'], sot.required_by)
        self.assertEqual(FAKE['resource_type'], sot.resource_type)
        self.assertEqual(FAKE['status'], sot.status)
        self.assertEqual(FAKE['status_reason'], sot.status_reason)
        self.assertEqual(FAKE['updated_time'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.telemetry.v2 import alarm

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'alarm_actions': ['1'],
    'alarm_id': IDENTIFIER,
    'combination_rule': {'alarm_ids': ['2', 'b'], 'operator': 'or', },
    'description': '3',
    'enabled': True,
    'insufficient_data_actions': ['4'],
    'name': '5',
    'ok_actions': ['6'],
    'project_id': '7',
    'repeat_actions': False,
    'severity': 'low',
    'state': 'insufficient data',
    'state_timestamp': '2015-03-09T12:15:57.233772',
    'timestamp': '2015-03-09T12:15:57.233772',
    'threshold_rule': {'meter_name': 'a',
                       'evaluation_periods:': '1',
                       'period': '60',
                       'statistic': 'avg',
                       'threshold': '92.6',
                       'comparison_operator': 'gt',
                       'exclude_outliers': True, },
    'time_constraints': [{'name': 'a', 'duration': 'b', 'start': 'c', }],
    'type': '10',
    'user_id': '11',
}


class TestAlarm(testtools.TestCase):

    def setUp(self):
        super(TestAlarm, self).setUp()
        self.resp = mock.Mock()
        self.resp.body = ''
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.sess = mock.Mock()
        self.sess.put = mock.Mock(return_value=self.resp)

    def test_basic(self):
        sot = alarm.Alarm()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/alarms', sot.base_path)
        self.assertEqual('metering', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_retrieve)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = alarm.Alarm(EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['alarm_actions'], sot.alarm_actions)
        self.assertEqual(IDENTIFIER, sot.alarm_id)
        self.assertEqual(EXAMPLE['combination_rule'], sot.combination_rule)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertTrue(sot.is_enabled)
        self.assertEqual(EXAMPLE['insufficient_data_actions'],
                         sot.insufficient_data_actions)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['ok_actions'], sot.ok_actions)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertFalse(sot.is_repeat_actions)
        self.assertEqual(EXAMPLE['severity'], sot.severity)
        self.assertEqual(EXAMPLE['state'], sot.state)
        self.assertEqual(EXAMPLE['state_timestamp'], sot.state_changed_at)
        self.assertEqual(EXAMPLE['timestamp'], sot.updated_at)
        self.assertEqual(EXAMPLE['threshold_rule'], sot.threshold_rule)
        self.assertEqual(EXAMPLE['time_constraints'], sot.time_constraints)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)

    def test_check_status(self):
        sot = alarm.Alarm(EXAMPLE)
        sot.check_state(self.sess)

        url = 'alarms/IDENTIFIER/state'
        self.sess.get.assert_called_with(url, endpoint_filter=sot.service)

    def test_change_status(self):
        sot = alarm.Alarm(EXAMPLE)
        self.assertEqual(self.resp.body, sot.change_state(self.sess, 'alarm'))

        url = 'alarms/IDENTIFIER/state'
        self.sess.put.assert_called_with(url, endpoint_filter=sot.service,
                                         json='alarm')



# -*- coding: utf-8 -*-

from ecl import service_filter


class SecurityOrderService(service_filter.ServiceFilter):
    """The security service."""

    valid_versions = [service_filter.ValidVersion('v1')]

    def __init__(self, version=None):
        """Create a security service."""
        super(SecurityOrderService, self).__init__(service_type='mss-rfg',
                                                   version=version)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.image import image_service
from ecl import resource2

class Tag(resource2.Resource):
    service = image_service.ImageService()
    base_path = '/' + service.version + '/images/%(image_id)s/tags/%(tag)s'

    def add_tag(self, session, image_id, tag):
        """Adds a specified tag to a specified image."""
        uri = self.base_path % {"image_id":image_id, "tag":tag}
        session.put(uri, endpoint_filter=self.service)

    def delete_tag(self, session, image_id, tag):
        """Delete a specified tag from a specified image"""
        uri = self.base_path % {"image_id": image_id, "tag": tag}
        session.delete(uri, endpoint_filter=self.service)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal import baremetal_service
from ecl import resource2


class AbsoluteLimits(resource2.Resource):

    maxPersonality = resource2.Body("maxPersonality")
    personality = resource2.Body("maxPersonality")

    maxPersonalitySize = resource2.Body("maxPersonalitySize")
    personality_size = resource2.Body("maxPersonalitySize")

    maxServerMeta = resource2.Body("maxServerMeta")
    server_meta = resource2.Body("maxServerMeta")

    maxTotalInstances = resource2.Body("maxTotalInstances", alternate_id=True)
    instances = resource2.Body("maxTotalInstances")

    totalInstancesUsed = resource2.Body("totalInstancesUsed")
    instances_used = resource2.Body("totalInstancesUsed")

    maxTotalKeypairs = resource2.Body("maxTotalKeypairs")
    keypairs = resource2.Body("maxTotalKeypairs")


class RateLimit(resource2.Resource):

    limits = resource2.Body("limit", type=list)
    regex = resource2.Body("regex")
    uri = resource2.Body("uri", alternate_id=True)


class Limits(resource2.Resource):
    base_path = "/limits"
    resource_key = "limits"
    service = baremetal_service.BaremetalService()

    allow_get = True

    absolute = resource2.Body("absolute", type=AbsoluteLimits)
    rate = resource2.Body("rate", type=list)

    def get(self, session, requires_id=False):
        """Get the Limits resource.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`

        :returns: A Baremetal Limits instance
        :rtype: :class:`~ecl.baremetal.v2.limits.Limits`
        """
        request = self._prepare_request(requires_id=False, prepend_key=False)

        response = session.get(request.uri, endpoint_filter=self.service)

        body = response.json()
        body = body[self.resource_key]

        absolute_body = self._filter_component(
            body["absolute"], AbsoluteLimits._body_mapping())
        self.absolute = AbsoluteLimits.existing(**absolute_body)

        rates_body = body["rate"]

        rates = []
        for rate_body in rates_body:
            rate_body = self._filter_component(rate_body,
                                               RateLimit._body_mapping())
            rates.append(RateLimit(**rate_body))

        self.rate = rates

        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import six
from ecl.tests.functional import base

class TestServerAction(base.BaseFunctionalTest):

    def test_01_start_server(self):
        server = self.conn.baremetal.start_server(
            "752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2",
            "DISK"
        )

    def test_02_stop_server(self):
        server = self.conn.baremetal.stop_server(
            "752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2",
            None
        )
        assert False

    def test_03_reboot_server(self):
       server = self.conn.baremetal.reboot_server(
           "752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2",
           "SOFT",
           "disk"
       )

    def test_04_get_management_console(self):
        server = self.conn.baremetal.get_management_console(
            "752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2",
            None
        )
        self.assertIsInstance(server.type, six.string_types)
        self.assertIsInstance(server.url, six.string_types)
        self.assertIsInstance(server.user_id, six.string_types)
        self.assertIsInstance(server.password, six.string_types)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.telemetry.v2 import statistics

EXAMPLE = {
    'aggregate': '1',
    'avg': '2',
    'count': '3',
    'duration': '4',
    'duration_end': '2015-03-09T12:45:00.000000',
    'duration_start': '2015-03-09T12:15:00.000000',
    'groupby': '7',
    'max': '8',
    'min': '9',
    'period': '10',
    'period_end': '2015-03-09T12:45:00.000000',
    'period_start': '2015-03-09T12:15:00.000000',
    'sum': '13',
    'unit': '14',
}


class TestStatistics(testtools.TestCase):

    def test_basic(self):
        sot = statistics.Statistics()
        self.assertEqual('statistics', sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/meters/%(meter_name)s/statistics', sot.base_path)
        self.assertEqual('metering', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_retrieve)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = statistics.Statistics(EXAMPLE)
        self.assertIsNone(sot.id)
        self.assertEqual(EXAMPLE['aggregate'], sot.aggregate)
        self.assertEqual(EXAMPLE['avg'], sot.avg)
        self.assertEqual(EXAMPLE['count'], sot.count)
        self.assertEqual(EXAMPLE['duration'], sot.duration)
        self.assertEqual(EXAMPLE['duration_end'], sot.duration_end_at)
        self.assertEqual(EXAMPLE['duration_start'], sot.duration_start_at)
        self.assertEqual(EXAMPLE['groupby'], sot.group_by)
        self.assertEqual(EXAMPLE['max'], sot.max)
        self.assertEqual(EXAMPLE['min'], sot.min)
        self.assertEqual(EXAMPLE['period'], sot.period)
        self.assertEqual(EXAMPLE['period_end'], sot.period_end_at)
        self.assertEqual(EXAMPLE['period_start'], sot.period_start_at)
        self.assertEqual(EXAMPLE['sum'], sot.sum)
        self.assertEqual(EXAMPLE['unit'], sot.unit)

    def test_list(self):
        sess = mock.Mock()
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=[EXAMPLE])
        sess.get = mock.Mock(return_value=resp)

        args = {'meter_name': 'example'}
        reply = statistics.Statistics.list(sess, path_args=args)

        url = '/meters/example/statistics'
        stat = next(reply)
        sess.get.assert_called_with(url, endpoint_filter=stat.service,
                                    params={})
        self.assertEqual(EXAMPLE, stat)
        self.assertRaises(StopIteration, next, reply)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl import exceptions


class Test_Exception(testtools.TestCase):
    def test_method_not_supported(self):
        exc = exceptions.MethodNotSupported(self.__class__, 'list')
        expected = ('The list method is not supported for ' +
                    'ecl.tests.unit.test_exceptions.Test_Exception')
        self.assertEqual(expected, str(exc))


class Test_HttpException(testtools.TestCase):

    def setUp(self):
        super(Test_HttpException, self).setUp()
        self.message = "mayday"

    def _do_raise(self, *args, **kwargs):
        raise exceptions.HttpException(*args, **kwargs)

    def test_message(self):
        exc = self.assertRaises(exceptions.HttpException,
                                self._do_raise, self.message)

        self.assertEqual(self.message, exc.message)

    def test_details(self):
        details = "some details"
        exc = self.assertRaises(exceptions.HttpException,
                                self._do_raise, self.message,
                                details=details)

        self.assertEqual(self.message, exc.message)
        self.assertEqual(details, exc.details)

    def test_http_status(self):
        http_status = 123
        exc = self.assertRaises(exceptions.HttpException,
                                self._do_raise, self.message,
                                http_status=http_status)

        self.assertEqual(self.message, exc.message)
        self.assertEqual(http_status, exc.http_status)



# -*- coding: utf-8 -*-

from ecl.compute import compute_service
from ecl import resource2


class Quota(resource2.Resource):
    resource_key = "quota_set"
    resources_key = "quota_sets"
    base_path = '/os-quota-sets'
    service = compute_service.ComputeService()
    
    # capabilities
    allow_get = True
    
    cores = resource2.Body("cores", int)
    fixed_ips = resource2.Body("fixed_ips", int)
    id = resource2.Body("id")
    injected_file_content_bytes = resource2.Body("injected_file_content_bytes", int)
    injected_file_path_bytes = resource2.Body("injected_file_path_bytes", int)
    injected_files = resource2.Body("injected_files", int)
    instances = resource2.Body("instances", int)
    key_pairs = resource2.Body("key_pairs", int)
    metadata_items = resource2.Body("metadata_items", int)
    ram = resource2.Body("ram", int)
    security_group_rules = resource2.Body("security_group_rules", int)
    security_groups = resource2.Body("security_groups", int)
    server_group_members = resource2.Body("server_group_members", int)
    server_groups = resource2.Body("server_groups", int)


class DefaultQuota(resource2.Resource):
    resource_key = "quota_set"
    resources_key = "quota_sets"
    base_path = '/os-quota-sets/%(tenant_id)s/defaults'
    service = compute_service.ComputeService()

    allow_get = True

    tenant_id = resource2.URI("tenant_id")
    cores = resource2.Body("cores", int)
    fixed_ips = resource2.Body("fixed_ips", int)
    id = resource2.Body("id")
    injected_file_content_bytes = resource2.Body("injected_file_content_bytes", int)
    injected_file_path_bytes = resource2.Body("injected_file_path_bytes", int)
    injected_files = resource2.Body("injected_files", int)
    instances = resource2.Body("instances", int)
    key_pairs = resource2.Body("key_pairs", int)
    metadata_items = resource2.Body("metadata_items", int)
    ram = resource2.Body("ram", int)
    security_group_rules = resource2.Body("security_group_rules", int)
    security_groups = resource2.Body("security_groups", int)
    server_group_members = resource2.Body("server_group_members", int)
    server_groups = resource2.Body("server_groups", int)

    def get(self, session, requires_id=False):
        return super(DefaultQuota, self).get(session, False)


class TenantUsage(resource2.Resource):
    resource_key = "tenant_usage"
    resources_key = "tenant_usages"
    base_path = '/os-simple-tenant-usage'
    service = compute_service.ComputeService()

    allow_get = True

    server_usages = resource2.Body("server_usages")
    start = resource2.Body("start")
    stop = resource2.Body("stop")
    tenant_id = resource2.Body("tenant_id")
    total_hours = resource2.Body("total_hours")
    total_local_gb_usage = resource2.Body("total_local_gb_usage")
    total_memory_mb_usage = resource2.Body("total_memory_mb_usage")
    total_vcpus_usage = resource2.Body("total_vcpus_usage")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.dns import dns_service
from ecl import resource2
from ecl import exceptions


class Zone(resource2.Resource):
    resource_key = None
    resources_key = "zones"
    base_path = '/v2/zones'
    service = dns_service.DnsService()

    # Capabilities
    allow_create = True
    allow_delete = True
    allow_update = True
    allow_list = True
    allow_get = True
    patch_update = True

    # Properties
    #: ID for the resource
    id = resource2.Body('id')
    #: ID for the pool hosting this zone. This parameter is not currently supported. It always returns an empty string.
    pool_id = resource2.Body('pool_id')
    #: ID for the project(tenant) that owns the resource.
    project_id = resource2.Body('project_id')
    #: DNS Name for the zone
    name = resource2.Body('name')
    #: e-mail for the zone. Used in SOA records for the zone. This parameter is not currently supported.
    #: It always returns an empty string.
    email = resource2.Body('email')
    #: TTL (Time to Live) for the zone. This parameter is not currently supported. it always return zero.
    ttl = resource2.Body('ttl')
    #: current serial number for the zone. This parameter is not currently supported. it always return zero.
    serial = resource2.Body('serial')
    #: status of the resource.
    status = resource2.Body('status')
    #: Description for this zone.
    description = resource2.Body('description')
    #: For secondary zones. The servers to slave from to get DNS information. This parameter is not currently supported.
    #: It always returns an empty string.
    masters = resource2.Body('masters')
    #: Type of zone. PRIMARY is controlled by ECL2.0 DNS, SECONDARY zones are slaved from another DNS Server.
    #: Defaults to PRIMARY. This parameter is not currently supported. It always returns an empty string.
    type = resource2.Body('type')
    #: For secondary zones. The last time an update was retrieved from the master servers.
    #: This parameter is not currently supported. it always return null.
    transferred_at = resource2.Body('transferred_at')
    #: Version of the resource. This parameter is not currently supported. it always return 1.
    version = resource2.Body('version')
    #: Date / Time when resource was created.
    created_at = resource2.Body('created_at')
    #: Date / Time when resource last updated.
    updated_at = resource2.Body('updated_at')
    #: Links to the resource, and other related resources. When a response has been broken into pages,
    #: we will include a next link that should be followed to retrive all results.
    links = resource2.Body('links')


    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))
